///////////////////////////////////////////////////////////////////////////
//
// VisualDSP++ 4.5 "Flash Programmer" flash driver for use to
// program SPI flash/EEPROM devices.
//
// (C) Copyright 2007 - Analog Devices, Inc.  All rights reserved.
//
// Basic understanding of the VisualDSP++ 4.5 flash programmer tool, also 
// called "plug-in", is required.
//
// Note: Due to the fact that the plug-in accepts to open intel hex files only
// 		 SPI programming works just for VisualDSP++ version 4.5 and higher!
//
// 		 Ensure that the option intel hex is chosen at the register LOAD
//       under Project Options!
// 
// In general, when the flash programming tool "Tools->Flash Programmer" 
// has been opened a driver file must be loaded. Loading a driver such as this
// one causes VisualDSP++ to upload the driver dxe file to the processor,
// to set a breakpoint at the label "AFP_BreakReady" and to start the processor.
// When the processor halts at the breakpoint the control went back to the 
// flash plug-in. In the box "Load File" a loader file "*.ldr" of a project
// must be selected to program a SPI device.
//
// Note: As a standard flash needs to be erased before programming please 
//       make use of the erase options in surrounding boxes. 
//
// Pressing the button "Load File" will cause the plug-in to access the loader 
// file and to fill the buffer "AFP_Buffer" with the first portion of data.
// Also, the number of words counted in bytes as well as the SPI start address 
// will be stored in certain variables.
//
// The contents of AFP_Buffer will be written to the SPI and sequentially the 
// AFP_Buffer will be filled with the next portion of the loader file. After 
// the whole loader file has been transferred to the SPI the processor halts 
// at the breakpoint as discussed above. 
//
// 
//////////////////////////////////////////////////////////////////////////////



#include "SPI_Device_Settings_W25X16.h"	// includes all the SPI specific settings.
#include "Errors.h"						// holds the errors the plug-in can solve.

#include <cdefBF537.h>
#include <stdio.h>
#include <ccblkfn.h>


/*-----------------------------------------------------------------------------
NOTE: 
The following defines are used by the plug-in. 
Deleting these defines can lead to Plug-in crashs. 
-----------------------------------------------------------------------------*/

#define TRUE			0x1
#define FALSE			0x0
#define BUFFER_SIZE		0x600
#define NUM_SECTORS		4
#define SECTORBASE		0


/*-----------------------------------------------------------------------------
SPI CONFIGURATION DEFINITIONS 
-----------------------------------------------------------------------------*/

#define COMMON_SPI_SETTINGS (SPE|MSTR)	// Configures the SPI in Master mode
#define TIMOD01 			(0x01)		// Makes SPI Transfer on TDBR write 
#define BAUD_RATE_DIVISOR 	133 		// SPI Clock = SCLK / (2 * BAUD_RATE_DIVISOR)


// Used with a single sector erase command
// The plug in will use this structure to display memory space.
typedef struct _SECTORLOCATION
{
	long lStartOff;
	long lEndOff;
	
}SECTORLOCATION;


/*-----------------------------------------------------------------------------
FLASH PROGRAMMER COMMANDS

The plug-in will send commands to this driver. 
This determines what function to call based on the command value.
The command-to-call assignments are listed below.

NOTE:
Changing the macros or values will lead to malfunction
-----------------------------------------------------------------------------*/

typedef enum
{
	NO_COMMAND,		// 0
	GET_CODES,		// 1
	RESET,			// 2
	WRITE,			// 3
	FILL,			// 4
	ERASE_ALL,		// 5
	ERASE_SECT,		// 6
	READ,			// 7
	GET_SECTNUM,	// 8
}enProgCmds;


/*-----------------------------------------------------------------------------
FUNCTION PROTOTYPES
-----------------------------------------------------------------------------*/
ERROR_CODE 	SetupForFlash	(void);
ERROR_CODE 	GetCodes		(void);
ERROR_CODE 	Wait_For_Status	(short);
ERROR_CODE 	Wait_For_nStatus(short);
ERROR_CODE 	ResetFlash		(void);
ERROR_CODE 	EraseFlash		(void);
ERROR_CODE 	EraseBlock		(int);
ERROR_CODE 	FillData		(unsigned long, long, long, int*);
ERROR_CODE 	ReadData		(unsigned long, long, long, int*);
ERROR_CODE 	WriteData		(unsigned long, long, long, int*);
ERROR_CODE 	GetSectorNumber	(unsigned long, int*);
ERROR_CODE 	WriteFlash		(unsigned long, long, long, int*, long*);

char 	ReadStatusRegister	(void);
void 	Wait_For_SPIF		(void);
void 	SetupSPI			(const int);
void 	SPI_OFF				(void);
void 	SendSingleCommand	(const short);
void 	Init_PLL			(void);	


/*-----------------------------------------------------------------------------
GLOBAL VARIABLES

For the plug-in's use
-----------------------------------------------------------------------------*/

char 			*AFP_Title = "ADSP-BF534/6/7 SPI Flash W25X16";	// This title will be displayed in the plug-in
char 			*AFP_Description = SPI_Description;		// The description will also be displayed in the plug-in. See the file "SPI_Device_Settings.h".
enProgCmds 		AFP_Command = NO_COMMAND;				// AFP_Command is dedicated to control program flow. Look at the switch case below.
int 			AFP_ManCode = 0x0;						// Some devices have Manufacturer codes to autodetect the device
int 			AFP_DevCode = 0x0;						// Some devices have device codes to autodetect the device
unsigned long 	AFP_Offset = 0;							// AFP_Offset holds the start address to the SPI device
int 			i, *AFP_Buffer;							// The plug-in copies the content of the *.ldr file to the buffer in small portions
long 			AFP_Size = BUFFER_SIZE;					// The AFP_Buffer size can be modified.
long 			AFP_Count = -1;							// The plug-in copies the number of transfers to be executed to the AFP_Count 
long 			AFP_Stride = -1;						// For the plug-in "Fill" function a stride can optionally be inserted
int 			AFP_NumSectors = SPI_SECTORS;			// The number of SPI sectors are given for sector erase function
long 			AFP_SectorSize = SPI_SECTOR_SIZE;		// The multiplier factor from sector to sector must be given
int 			AFP_Sector = 0;							// In case of sector erase the right sector must be chosen and hold in here
int 			AFP_Error = 0;							// contains last error encountered
bool 			AFP_Verify = FALSE;						// verify writes or not
long 			AFP_StartOff = 0x0;						// sector start offset
long 			AFP_EndOff = 0x0;						// sector end offset
int				AFP_FlashWidth = 0x8;					// width of the flash device taken
int 			*AFP_SectorInfo;
SECTORLOCATION 	SectorInfo[NUM_SECTORS];
bool 			bExit = FALSE; 							// exit flag


/*-----------------------------------------------------------------------------
NOTES ON AFP BUFFER
-----------------------------------------------------------------------------*/


// By making AFP_Buffer as big as possible the plug-in can send and
// receive more data at a time making the data transfer quicker.
//
// By allocating it on the heap the compiler does not create an
// initialized array therefore making the driver image smaller
// and faster to load.
//
// We have modified the linker description file (LDF) so that the heap
// is large enough to store BUFFER_SIZE elements at this point
// 
// The AFP_Buffer gets filled by the programmer plug-in. The data the plug-in 
// fetches from the *.ldr file will be stored in the AFP_Buffer byte wise to 
// avoid gaps in the buffer.
// 
// The verify command uses the buffer to store the data read from the SPI 
// flash for comparison. The fill command uses the first address of the buffer
// to hold the value to be filled.
// 
// - In case of loading the code to the SPI flash the plug-in will fill the 
//   buffer automatically so that this driver just has to fetch the data from 
//   the buffer and transfer it to the SPI flash.
// - In case of reading the data from the SPI flash (plug-in command "VERIFY") 
//   this driver just has to fetch the data from the SPI flash and store it 
//   in the buffer.

/*-----------------------------------------------------------------------------
MAIN
-----------------------------------------------------------------------------*/

main()
{
	int i = 0;
	
	AFP_Buffer = (int *)malloc(BUFFER_SIZE);

 // Check if space allocated for the buffer
	if ( AFP_Buffer == NULL )
	{
	 // Buffer Not Initialized
		AFP_Error = BUFFER_IS_NULL;
	}

	Init_PLL();	
	GetCodes();	

 // Points to the beginning of the sector table.
	AFP_SectorInfo = (int*)&SectorInfo[0];	


 // Plug in will wait at "AFP_BreakReady" until the next command is issued. 	
	while (!bExit)
	{
	 // Waits for command.
		asm("AFP_BreakReady:");	
				
		if (FALSE)
			asm("jump AFP_BreakReady;");

	 // Determine function call based on command value
		switch (AFP_Command)
		{
		 // GET CODES
			case GET_CODES:
				AFP_Error = GetCodes();
				break;

		 // RESET (Write Disable)
			case RESET:
				AFP_Error = ResetFlash();
				break;

		 // WRITE DATA (Program the flash)
			case WRITE:
				AFP_Error = WriteData(AFP_Offset, AFP_Count, AFP_Stride, AFP_Buffer);
				break;

		 // FILL FLASH
			case FILL:
				AFP_Error = FillData(AFP_Offset, AFP_Count, AFP_Stride, AFP_Buffer);
				break;

		 // CHIP ERASE ("Erase" button on plug-in)
			case ERASE_ALL:
				AFP_Error = EraseFlash();
				break;

		 // SECTOR ERASE (Erases Sector #[AFP_Sector] determined from GET_SECTNUM)
			case ERASE_SECT:
				AFP_Error = EraseBlock(AFP_Sector);
				break;

		 // READ DATA ("Verify" button)
			case READ:
				AFP_Error = ReadData(AFP_Offset, AFP_Count, AFP_Stride, AFP_Buffer);
				break;

		 // GET SECTOR NUMBER (Used with the Erase Sector function)
			case GET_SECTNUM:
				AFP_Error = GetSectorNumber(AFP_Offset, &AFP_Sector);
				break;

		 // NO COMMAND/UNKNOWN COMMAND (Set Error if No/Unknown Command)
			case NO_COMMAND:
			default:
				AFP_Error = UNKNOWN_COMMAND;
				break;
		}

	 // Clear the Command
		AFP_Command = NO_COMMAND;
	}

 // Deallocate the Buffer
	if ( AFP_Buffer )
		free( AFP_Buffer );

	return 0;
}



///////////////////////////////////////////////////////////////////////////////
// ERROR_CODE WriteData()
//
// Write the contents of AFP_Buffer to SPI flash.
//
// Inputs:	unsigned long ulStart - SPI start address 			[AFP_Offset]
//			long lCount - number of elements write to the SPI 	[AFP_Count]
//			long lStride - address increment to the SPI			[AFP_Stride]
//			int *pnData - pointer to data buffer				[AFP_Buffer]
//
// This function takes the input parameters and calls the function 
// WriteFlash which writes the data of AFP_Buffer to the SPI.
//
// The function WriteFlash will usually be executed multiple times because 
// the SPI needs a break after a SPI page has been filled.
// After each page transfer the addresses and counts need to be recalculated .
///////////////////////////////////////////////////////////////////////////////
ERROR_CODE WriteData(unsigned long ulStart, long lCount, long lStride, int *pnData)
{

	unsigned long ulWStart = ulStart; 
	long lWCount = lCount, lWriteCount;
	long *pnWriteCount = &lWriteCount;
	
	
	ERROR_CODE ErrorCode = NO_ERR;

	while (lWCount != 0)	
	{
		ErrorCode = WriteFlash(ulWStart, lWCount, lStride, pnData, pnWriteCount);
		
	 // Adjust counter
		lWCount -= *pnWriteCount;

	 // Recalculate address pointers.
		ulWStart += *pnWriteCount;

		pnData += *pnWriteCount/4;
	}

 // Return the appropriate error code
	return ErrorCode;
	
}


///////////////////////////////////////////////////////////////////////////////
// ERROR_CODE FillData()
//
// Fill flash with a value.
//
// Inputs:	unsigned long ulStart - SPI start address 			[AFP_Offset]
//			long lCount - number of elements write to the SPI 	[AFP_Count]
//			long lStride - address increment to the SPI			[AFP_Stride]
//			int *pnData - pointer to data buffer				[AFP_Buffer]
//
///////////////////////////////////////////////////////////////////////////////
ERROR_CODE FillData(unsigned long ulStart, long lCount, long lStride, int *pnData)
{

	
	unsigned long ulWAddr = ulStart; 
	long lWCount = 1, lWriteCount;
	long *pnWriteCount = &lWriteCount;
	int i;
	
	ERROR_CODE ErrorCode = NO_ERR;

	
    for (i = 0; i < lCount; i++, ulWAddr += lStride)
	{
		ErrorCode = WriteFlash(ulWAddr, lWCount, lStride, pnData, pnWriteCount);
	}
	
 // Return the appropriate error code
	return ErrorCode;
	
}


//////////////////////////////////////////////////////////////
// ERROR_CODE WriteFlash()
//
// Write up to the maximum of SPI_PAGE_SIZE bytes to flash
//
// Inputs:	unsigned long ulStartAddr - address to write to
//			int iDataSource - pointer to the AFP_Buffer
//			long lTransferCount - number of elements to send
//////////////////////////////////////////////////////////////

ERROR_CODE WriteFlash ( unsigned long ulStartAddr, long lTransferCount, long lModify, int *iDataSource, long *lWriteCount )
{
	unsigned long ulWAddr;
	long lWTransferCount = 0;
	int i;
	char iData;
	int dummyread;
	ERROR_CODE ErrorCode = NO_ERR;

 // First, a Write Enable Command must be sent to the SPI.
	SendSingleCommand(SPI_WREN);
 
 // Second, the SPI Status Register will be tested whether WEL has been set. 
	ErrorCode = Wait_For_nStatus(WEL);

	if( POLL_TIMEOUT == ErrorCode )
		return ErrorCode;
	else

 // Third, the address will be shifted out the SPI MOSI bytewise.
 
 // Turns the SPI on
	SetupSPI( (COMMON_SPI_SETTINGS|TIMOD01) );
	
 // Sends Command
	*pSPI_TDBR = SPI_PP;
	Wait_For_SPIF();
	
	#ifdef SPI_24BIT_ADDRESSING

	 // Sends bits [23:16] of address
		ulWAddr = (ulStartAddr >> 16);
		*pSPI_TDBR = ulWAddr;	
		Wait_For_SPIF();

	#endif

	#if	defined (SPI_24BIT_ADDRESSING) || defined (SPI_16BIT_ADDRESSING)

 	 // Sends bits [15:8] of address
		ulWAddr = (ulStartAddr >> 8);
		*pSPI_TDBR = ulWAddr;	
		Wait_For_SPIF();
	
	#endif
	
 // Sends bits [7:0] of address
	ulWAddr = ulStartAddr;
	*pSPI_TDBR = ulWAddr;	
	Wait_For_SPIF();
	
 // Dummy Read to empty RDBR
	dummyread = *pSPI_RDBR;

	
 // Fourth, lTransferCount bytes will be transferred
 // Maximum number of bytes: SPI_PAGE_SIZE
	for (i=0; (i < lTransferCount) && (i < SPI_PAGE_SIZE); i++, lWTransferCount++)
	{
		iData = *((char *)iDataSource);	
		*pSPI_TDBR = iData;
		Wait_For_SPIF();

		((char *)iDataSource)++;
	}
	
 // Turns the SPI off
	SPI_OFF();

	
 // Sixth, the SPI Write in Progress (BUSY) Bit must be polled to ensure the 
 // programming is done before start of next transfer.
	ErrorCode = Wait_For_Status(WIP);
		
	if( POLL_TIMEOUT == ErrorCode )
		return ErrorCode;
	else

	*lWriteCount = lWTransferCount;
	
	return ErrorCode;
}

//////////////////////////////////////////////////////////////
// ERROR_CODE ReadData()
//
// Read a value from flash for verify purpose
//
// Inputs:	unsigned long ulStart - holds the SPI start address
//			int pnData - pointer to store value read from flash
//			long lCount - number of elements to read
//////////////////////////////////////////////////////////////

ERROR_CODE ReadData(unsigned long ulStart, long lCount, long lStride, int *pnData)
{

	unsigned long ShiftValue;
	char dummyread, *cnData,ReadValue;
	int i;

 // Pointer cast to be able to increment byte wise
	cnData = (char *)pnData; 

 // Configure the SPI
	SetupSPI((COMMON_SPI_SETTINGS|TIMOD01));

 // Send command
	*pSPI_TDBR = SPI_READ;			
	Wait_For_SPIF();				

	#ifdef SPI_24BIT_ADDRESSING

	 // Sends bits [23:16] of address
		ShiftValue = (ulStart >> 16);
		*pSPI_TDBR = ShiftValue;		// Send the byte to the SPI device
		Wait_For_SPIF();				// Wait until the instruction has been sent

	#endif

	#if	defined (SPI_24BIT_ADDRESSING) || defined (SPI_16BIT_ADDRESSING)

	 // Sends bits [15:8] of address
		ShiftValue = (ulStart >> 8);
		*pSPI_TDBR = ShiftValue;		
		Wait_For_SPIF();

	#endif

 // Sends bits [7:0] of address
	*pSPI_TDBR = ulStart;
	Wait_For_SPIF();
	
 // Dummy read to empty RDBR		
	dummyread = *pSPI_RDBR;


 // Receive data from MISO
	for (i=0; i<lCount; i++)
	{
	 // Dummy write to fill RDBR
		*pSPI_TDBR = 0;
		Wait_For_SPIF();
		
	 // Copy byte from RDBR
		ReadValue  = *pSPI_RDBR;
		
	 // Store byte into buffer
		*cnData = ReadValue;

	 // Increment the pointer of the buffer
		cnData++;
	}
	
 // Turn the SPI off
	SPI_OFF();
	

	return NO_ERR;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE EraseFlash()
//
// Sends an "erase all" command to the flash.
//
//////////////////////////////////////////////////////////////

ERROR_CODE EraseFlash()
{

	ERROR_CODE ErrorCode = NO_ERR;
		
 // Send Write Enable command
	SendSingleCommand(SPI_WREN);
	
 // Verify WREN bit on SPI Status Register has been set 
	ErrorCode = Wait_For_nStatus(WEL);

	if( POLL_TIMEOUT == ErrorCode )
	{
		return ErrorCode;
	}
	else

 // Send Chip Erase command
	SendSingleCommand(SPI_BE);
	
 // Poll Write in Progress (BUSY) Bit of Status Register
 // until erase is complete
	ErrorCode = Wait_For_Status(WIP);
	
	return ErrorCode;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE EraseBlock()
//
// Sends an "erase block" command to the flash.
//
//////////////////////////////////////////////////////////////

ERROR_CODE EraseBlock( int nBlock )
{
	unsigned long ulSectorOff = 0x0, ShiftValue;
	ERROR_CODE ErrorCode = NO_ERR;	


 // Check for boundary error.
	if ( (nBlock < 0) || (nBlock > AFP_NumSectors) )
	{
		ErrorCode = INVALID_BLOCK;
		return ErrorCode;
	}
	
 // Determine Block Offset
	if ( (nBlock >= 0) && (nBlock < AFP_NumSectors) )
	{
		ulSectorOff = (nBlock * AFP_SectorSize);
	}
	
	else
	{
		ErrorCode = INVALID_BLOCK;
		return ErrorCode;
	}

 // Send Write Enable command
	SendSingleCommand(SPI_WREN);
	
 // Verify WREN bit on SPI Status Register has been set 
	ErrorCode = Wait_For_nStatus(WEL);
	
	if( POLL_TIMEOUT == ErrorCode )
	{
		return ErrorCode;
	}
	else

	
 // Configure the SPI
	SetupSPI( (COMMON_SPI_SETTINGS|TIMOD01) );

 // Send Erase Block command	
	*pSPI_TDBR = SPI_SE;
	Wait_For_SPIF();

	#ifdef SPI_24BIT_ADDRESSING
	 
	 // Sends bits [23:16] of address
		ShiftValue = (ulSectorOff >> 16);
		*pSPI_TDBR = ShiftValue;			
		Wait_For_SPIF();

	#endif

	#if	defined (SPI_24BIT_ADDRESSING) || defined (SPI_16BIT_ADDRESSING)

	 // Sends bits [15:8] of address
		ShiftValue = (ulSectorOff >> 8);
		*pSPI_TDBR = ShiftValue;
		Wait_For_SPIF();

	#endif

 // Sends bits [7:0] of address
	*pSPI_TDBR = ulSectorOff;
	Wait_For_SPIF();					
	
 // Turns off the SPI
	SPI_OFF();
	Wait_For_SPIF();

 // Poll Write in Progress (BUSY) Bit of Status Register
 // until erase is complete
	ErrorCode = Wait_For_Status(WIP);

	return ErrorCode;
}




//////////////////////////////////////////////////////////////
// ERROR_CODE GetCodes()
//
// Sector Table Setup for a certain device
//
// Macros used defined in "SPI_Device_Settings.h"
//////////////////////////////////////////////////////////////

ERROR_CODE GetCodes()
{
	
 // In regards to the device chosen the sectors will be organized.
	AFP_NumSectors = SPI_SECTORS;
	AFP_SectorSize = SPI_SECTOR_SIZE;
	
	int i;
	
	for (i = 0; i < SPI_SECTORS; ++i)
	{
		SectorInfo[i].lStartOff = /*SECTORBASE + */(i * SPI_SECTOR_SIZE);
		SectorInfo[i].lEndOff 	= /*SECTORBASE + */((i + 1) * SPI_SECTOR_SIZE) - 1;
	}
	
	return NO_ERR;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE GetSectorNumber()
//
// Gets a sector number based on the offset.
// 
// Macros used can be found in "SPI_Device_Settings.h"
//////////////////////////////////////////////////////////////

ERROR_CODE GetSectorNumber( unsigned long ulOffset, int *pnSector )
{

	int i;
	ERROR_CODE ErrorCode = NO_ERR;

	// This function makes use of the settings in the file "SPI_Device_Settings.h".

	// Determine the Sector
	if(ulOffset > SPI_SECTOR_SIZE * SPI_SECTORS - 1)
	{
		ErrorCode = INVALID_SECTOR;
		return ErrorCode;
	}

	for (i=0; i < SPI_SECTORS; i++)
	{
		if (ulOffset >  SectorInfo[i].lStartOff && ulOffset < SectorInfo[i].lEndOff)
		{
			*pnSector = i; 
			break;
		}
	}


	return ErrorCode;

}

//////////////////////////////////////////////////////////////
// ERROR_CODE ResetFlash()
//
// This function sends a Write Disable Command to the SPI device
// After that it checks the Write Enable Latch.
//
//////////////////////////////////////////////////////////////

ERROR_CODE ResetFlash()
{
	ERROR_CODE ErrorCode = NO_ERR;	
	
 // Send Write Disable Command
	SendSingleCommand(SPI_WRDI);

 // Verify that WEL has been cleared
	ErrorCode = Wait_For_Status(WEL);
	
	return ErrorCode;
}


//////////////////////////////////////////////////////////////
// char ReadStatusRegister()
//
// Returns the 8-bit value of the status register.
// Inputs - none
// returns- second location of status_register[2],
//         first location is garbage.
// Core sends the command
//
//////////////////////////////////////////////////////////////
char ReadStatusRegister(void)
{
	char dummyread, status_register = 0xFF;
	int n;

 // Configure the SPI
	SetupSPI( (COMMON_SPI_SETTINGS|TIMOD01) );

 // Send Read Status Command
	*pSPI_TDBR = SPI_RDSR;
	Wait_For_SPIF();
	
 // Dummy read to empty RDBR
	dummyread = *pSPI_RDBR;
	
 // Dummy write to fill RDBR with Status Register
	*pSPI_TDBR = 0;			
	Wait_For_SPIF();
	
 // Copy the status register to local variable
	status_register = *pSPI_RDBR;
	
 // Turn off the SPI
	SPI_OFF();

	return status_register;
}

//////////////////////////////////////////////////////////////
// void Wait_For_SPIF()
//
// Polls the SPIF (SPI single word transfer complete) bit
// of SPISTAT until the transfer is complete.
// Inputs - none
// returns- none
//
//////////////////////////////////////////////////////////////
void Wait_For_SPIF(void)
{
	int n;
	
 // Run a 1000 core clock cycle delay
	for(n=0; n < 1000; n++)
		asm("nop;");
		
 // Loop until SPIF is set
	while(1)
	{
		unsigned short iTest = *pSPI_STAT;
		
		if( (iTest & SPIF) )
			break;
	}
}


//////////////////////////////////////////////////////////////
// ERROR_CODE Wait_For_nStatus()
//
// Polls bit in question of the Flash's status register until
// it has been set.
// 
// Inputs - Statusbit (bit in question)
//
//////////////////////////////////////////////////////////////

ERROR_CODE Wait_For_nStatus(short Statusbit)
{
	int n, i;
	char status_register = 0;
	ERROR_CODE ErrorCode = NO_ERR;
	
 // Polls for 500 cycles, then deems timeout
	for(i = 0; i < 500; i++)
	{
		status_register = ReadStatusRegister();
		
		if((status_register & Statusbit))
		{
			ErrorCode = NO_ERR;
			return ErrorCode;
		}
		
	 // Timeout error
		ErrorCode = POLL_TIMEOUT;
	}
					
	return ErrorCode;
}

//////////////////////////////////////////////////////////////
// ERROR_CODE Wait_For_Status()
//
// Polls bit in question of the Flash's status register until
// it has been cleared. 
// 
// Inputs - Statusbit (bit in question)
//
//////////////////////////////////////////////////////////////
ERROR_CODE Wait_For_Status( short Statusbit )
{
	int n, i;
	char status_register = 0xFF;
	ERROR_CODE ErrorCode = NO_ERR;
	
		while(1)
		{
			status_register = ReadStatusRegister();
			
			if( !(status_register & Statusbit) )
			{
				ErrorCode = NO_ERR;
				break;
			}

			ErrorCode = POLL_TIMEOUT;
		}

	
	return ErrorCode;
	
}

//////////////////////////////////////////////////////////////
// void SendSingleCommand()
//
// Sends a single command to the SPI flash
// inputs - the 8-bit command to send
// returns- none
//
//////////////////////////////////////////////////////////////
void SendSingleCommand( const short iCommand )
{
	int n;

	//turns on the SPI in single write mode
	SetupSPI( (COMMON_SPI_SETTINGS|TIMOD01) );

	//sends the actual command to the SPI TX register
	*pSPI_TDBR = iCommand;

	//The SPI status register will be polled to check the SPIF bit
	Wait_For_SPIF();

	//The SPI will be turned off
	SPI_OFF();

	
}

//////////////////////////////////////////////////////////////
// void SetupSPI()
// 
// 
// Sets up the SPI for mode specified in spi_setting
// 
// Inputs - spi_setting
// 			Refer to the SPI_CTL register description in the
// 			HRM for configuration details
//////////////////////////////////////////////////////////////
void SetupSPI( const int spi_setting )
{
	
 // Set SPI pins for non-GPIO mode
 // Configure FER to be peripheral control
 	*pPORTF_FER |= PF11 | PF12 | PF13;    

 // Sets up the PF to be the slave select of the SPI	
 // Blackfin PF pin must be set as output
	*pPORTFIO_DIR |= PF10;
	
 // Asserts the PF dedicated as SPI select
 // SPI must detect a high-to-low transition
	*pPORTFIO_SET |= PF10;
	
	for(i=0; i<2000; i++)
	{
		asm("nop;");
	}
	
	*pPORTFIO_CLEAR |= PF10;
	
	*pSPI_BAUD = BAUD_RATE_DIVISOR;
	*pSPI_CTL = spi_setting;

}

//////////////////////////////////////////////////////////////
// void SPI_OFF()
// 
// Turns off the SPI
// 
// Inputs - none
//////////////////////////////////////////////////////////////

void SPI_OFF(void)
{
	int i;
	
 // Reset SPI Registers (disables SPI)
	*pSPI_CTL = 0x0400;
	*pSPI_FLG = 0;
	*pSPI_BAUD = 0;


 // Disables the PF dedicated as SPI select
	*pPORTFIO_SET |= PF10;
			
	for(i=0; i<2000; i++)
		asm("nop;");
	
}
